import 'dart:convert';

import 'package:flutter_view_model/utils/string_utils.dart';

/// Decodes a JWT Token into the content property
class JwtDecoder {
  // Properties

  final Map<String, dynamic> content;

  // Methods
  JwtDecoder(String jwt)
      : assert(!isEmpty(jwt)),
        content = _parseJwt(jwt);

  static Map<String, dynamic> _parseJwt(String token) {
    final parts = token.split('.');
    if (parts.length != 3) {
      throw Exception('invalid token');
    }

    final payload = _decodeBase64(parts[1]);
    final payloadMap = json.decode(payload);
    if (payloadMap is! Map<String, dynamic>) {
      throw Exception('invalid jwt payload');
    }

    return payloadMap;
  }

  static String _decodeBase64(String str) {
    String output = str.replaceAll('-', '+').replaceAll('_', '/');

    switch (output.length % 4) {
      case 0:
        break;
      case 2:
        output += '==';
        break;
      case 3:
        output += '=';
        break;
      default:
        throw Exception('Illegal base64url string!"');
    }

    return utf8.decode(base64Url.decode(output));
  }
}
